from dataclasses import dataclass

from lxml import etree


@dataclass
class Cuenta:
    nombre: str
    tipo: str
    banco: str
    numero: str
    rfc: str

    @classmethod
    def from_etree(cls, element: etree._Element):
        cuenta = cls(
            nombre=element.get('Nombre'),
            tipo=element.get('TipoCuenta'),
            banco=element.get('BancoEmisor') or element.get('BancoReceptor'),
            numero=element.get('Cuenta'),
            rfc=element.get('RFC'),
        )
        return cuenta



import datetime

import pytest

from cep import Client, Cuenta, Transferencia


@pytest.fixture
def client():
    yield Client()


@pytest.fixture
def transferencia():
    yield Transferencia(
        fecha_operacion=datetime.datetime(2019, 4, 12, 13, 31, 44),
        ordenante=Cuenta(
            nombre='Matin Tamizi',
            tipo='40',
            banco='STP',
            numero='646180157042875763',
            rfc='ND',
        ),
        beneficiario=Cuenta(
            nombre='MATIN TAMIZI',
            tipo='40',
            banco='BBVA BANCOMER',
            numero='012180004643051249',
            rfc='TAMA840916669',
        ),
        monto=8.17,
        clave_rastreo='CUENCA1555093850',
        concepto='Matin',
        emisor='90646',
        receptor='40012',
        sello=(
            'X8YFvAfKZhV72datpHzKes/AaOyLqgs0uDWlVqrDy8i0FV96ajZY17Hz9X35c7'
            'z/TrSSvw6BQiqVWbJGG5xriNn8PK4pFKF6nyCEr6uGQ6FuF7YqAD6tUK55BBKT'
            'dqF3j+qummKguTHJyttR4xMwmOpiuwkgXuUFaEEHiO+UjgIk7BVzkULkZdpciL'
            'rY4czMZhdqpQ7if0udu2BxWI99eU9ZqaAtILyt39MtCPObu61D4A6SFnw6JwsU'
            'Rm2wCZ4KSYzex18Re3Hrg+BLri5drlgcPSG5/OBeE2omlcuZTQqd5iUzRt/XVg'
            '33arK4M8h2hbcfU/xwtYEDBQ6Jewh+tg=='
        ),
    )



import datetime as dt
import os

import pytest

from cep import Transferencia


@pytest.mark.vcr
def test_validar_transferencia(transferencia):
    tr = Transferencia.validar(
        fecha=dt.date(2019, 4, 12),
        clave_rastreo='CUENCA1555093850',
        emisor='90646',  # STP
        receptor='40012',  # BBVA
        cuenta='012180004643051249',
        monto=8.17,
    )
    assert tr == transferencia
    assert type(tr.to_dict()) is dict


@pytest.mark.vcr
def test_fail_validar_transferencia():
    tr = Transferencia.validar(
        fecha=dt.date(2019, 1, 1),
        clave_rastreo='guey',
        emisor='90646',
        receptor='40012',
        cuenta='012180004643051249',
        monto=1111111.00,
    )
    assert tr is None


@pytest.mark.vcr
def test_descarga_pdf(transferencia):
    pdf = transferencia.descargar()
    file_dir = os.path.dirname(__file__)
    file_path = os.path.join(file_dir, 'CEP-20190412-CUENCA1555093850.pdf')
    with open(file_path, 'rb') as f:
        assert pdf == f.read()


@pytest.mark.vcr
def test_descagar_transferencia_con_fecha_distinta(transferencia):
    tr = Transferencia.validar(
        fecha=dt.date(2019, 8, 29),
        clave_rastreo='MBAN01001908300003463991',
        emisor='40012',  # BBVA
        receptor='90646',  # STP
        cuenta='646180157048010399',
        monto=300,
    )
    assert type(tr.to_dict()) is dict
    tr.descargar()



import datetime
from dataclasses import asdict, dataclass
from typing import Optional

import clabe
from lxml import etree

from .client import Client
from .cuenta import Cuenta


@dataclass
class Transferencia:
    fecha_operacion: datetime.datetime
    ordenante: Cuenta
    beneficiario: Cuenta
    monto: float
    concepto: str
    clave_rastreo: str
    emisor: str
    receptor: str
    sello: str

    @classmethod
    def validar(
        cls,
        fecha: datetime.date,
        clave_rastreo: str,
        emisor: str,
        receptor: str,
        cuenta: str,
        monto: float,
    ):
        client = cls._validar(
            fecha, clave_rastreo, emisor, receptor, cuenta, monto
        )
        if not client:
            return None
        xml = cls._descargar(client, 'XML')
        resp = etree.fromstring(xml)

        ordenante = Cuenta.from_etree(resp.find('Ordenante'))
        beneficiario = Cuenta.from_etree(resp.find('Beneficiario'))
        concepto = resp.find('Beneficiario').get('Concepto')
        fecha_operacion = datetime.datetime.fromisoformat(
            str(fecha) + ' ' + resp.get('Hora')
        )
        transferencia = cls(
            fecha_operacion=fecha_operacion,
            ordenante=ordenante,
            beneficiario=beneficiario,
            monto=monto,
            concepto=concepto,
            clave_rastreo=clave_rastreo,
            emisor=emisor,
            receptor=receptor,
            sello=resp.get('sello'),
        )
        setattr(transferencia, '__client', client)
        return transferencia

    def descargar(self, formato: str = 'PDF') -> bytes:
        """formato puede ser PDF, XML o ZIP"""
        client = getattr(self, '__client', None)
        if not client:
            client = self._validar(
                self.fecha_operacion.date(),
                self.clave_rastreo,
                self.emisor,
                self.receptor,
                self.beneficiario.numero,
                self.monto,
            )
        return self._descargar(client, formato)

    def to_dict(self) -> dict:
        return asdict(self)

    @staticmethod
    def _validar(
        fecha: datetime.date,
        clave_rastreo: str,
        emisor: str,
        receptor: str,
        cuenta: str,
        monto: float,
    ) -> Optional[Client]:
        assert emisor in clabe.BANKS.values()
        assert receptor in clabe.BANKS.values()
        client = Client()  # Use new client to ensure thread-safeness
        request_body = dict(
            fecha=fecha.strftime('%d-%m-%Y'),
            criterio=clave_rastreo,
            emisor=emisor,
            receptor=receptor,
            cuenta=cuenta,
            monto=monto,
        )
        resp = client.post('/valida.do', request_body)
        # None si no pudó validar
        return client if b'no encontrada' not in resp else None

    @staticmethod
    def _descargar(client: Client, formato: str = 'PDF') -> bytes:
        """formato puede ser PDF, XML o ZIP"""
        return client.get(f'/descarga.do?formato={formato}')



import requests

USER_AGENT = (
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 '
    '(KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36'
)


class Client:
    base_url = 'http://www.banxico.org.mx/cep'

    def __init__(self):
        self.session = requests.Session()
        self.session.headers['User-Agent'] = USER_AGENT
        self.base_data = dict(
            tipoCriterio='T',
            receptorParticipante=0,
            captcha='c',
            tipoConsulta=1,
        )

    def get(self, endpoint: str, **kwargs) -> bytes:
        return self.request('get', endpoint, {}, **kwargs)

    def post(self, endpoint: str, data: dict, **kwargs) -> bytes:
        data = {**self.base_data, **data}
        return self.request('post', endpoint, data, **kwargs)

    def request(
        self, method: str, endpoint: str, data: dict, **kwargs
    ) -> bytes:
        url = self.base_url + endpoint
        response = self.session.request(method, url, data=data, **kwargs)
        if not response.ok:
            response.raise_for_status()
        return response.content



from importlib.machinery import SourceFileLoader

from setuptools import find_packages, setup

version = SourceFileLoader('version', 'cep/version.py').load_module()


with open('README.md', 'r') as f:
    long_description = f.read()


setup(
    name='cepmex-bp',
    version=version.__version__,
    author='BanPAY',
    author_email='dev@banpay.com',
    description='CEP client library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/lucf25/cep-python',
    packages=find_packages(),
    include_package_data=True,
    package_data=dict(cep=['py.typed']),
    python_requires='>=3.7',
    install_requires=[
        'requests>=2.25,<2.26',
        'clabe-bp>=1.2.4,<1.3',
        'lxml>=4.6.2,<4.7',
        'dataclasses>=0.6;python_version<"3.7"',
    ],
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)
